/*---------------------------------------------------------------------------
Copyright (C) GeoLib.
This code is used under license from GeoLib (www.geolib.co.uk). This or
any modified versions of this cannot be resold to any other party.
---------------------------------------------------------------------------*/


/**--------------------------------------------------------------------------<BR>
\file GeoDraw.cpp
\brief Implementation file for the CGeoDraw class. 
<P>---------------------------------------------------------------------------*/

#include "StdAfx.h"	// local StdAfx (replace / delete as necassary)
#include "GeoDraw.h"
#include "C2DPoint.h"
#include "C2DLine.h"
#include "C2DArc.h"
#include "C2DRect.h"
#include "C2DCircle.h"
#include "C2DPolyBase.h"
#include "C2DSegment.h"
#include "C2DTriangle.h"
#include "C2DHoledPolyBase.h"
#include <afxwin.h> // Drawing


#include "C2DHoledPolygonSet.h"





/**--------------------------------------------------------------------------<BR>
CGeoDraw::CGeoDraw
\brief Constructor.
<P>---------------------------------------------------------------------------*/
CGeoDraw::CGeoDraw(void)
{
	m_pOffset = new C2DPoint(0, 0);
	m_pScale = new C2DPoint(1, 1);
}

/**--------------------------------------------------------------------------<BR>
CGeoDraw::CGeoDraw
\brief Destructor.
<P>---------------------------------------------------------------------------*/
CGeoDraw::~CGeoDraw(void)
{
	delete m_pOffset;
	delete m_pScale;
}

void CGeoDraw::operator=(CGeoDraw& rOther)
{
	*m_pOffset = *rOther.m_pOffset;
	*m_pScale = *rOther.m_pScale;
}

/**--------------------------------------------------------------------------<BR>
CGeoDraw::SetScale
\brief Sets the scale.
<P>---------------------------------------------------------------------------*/
void CGeoDraw::SetScale(const C2DPoint& pt)
{
	(*m_pScale) = pt;

}

/**--------------------------------------------------------------------------<BR>
CGeoDraw::SetOffset
\brief Sets the offset.
<P>---------------------------------------------------------------------------*/
void CGeoDraw::SetOffset(const C2DPoint& pt)
{
	(*m_pOffset) = pt;
}

/**--------------------------------------------------------------------------<BR>
CGeoDraw::GetScale
\brief Gets the scale.
<P>---------------------------------------------------------------------------*/
const C2DPoint& CGeoDraw::GetScale(void) const
{
	return (*m_pScale);

}

/**--------------------------------------------------------------------------<BR>
CGeoDraw::GetOffSet
\brief Gets the offset.
<P>---------------------------------------------------------------------------*/
const C2DPoint& CGeoDraw::GetOffSet(void) const
{
	return (*m_pOffset);
}


/**--------------------------------------------------------------------------<BR>
CGeoDraw::DrawPoints
\brief DrawPoints.
<P>---------------------------------------------------------------------------*/
void CGeoDraw::DrawPoints(const C2DPolyBase& Poly, CDC* pDC, unsigned long colorRef) const
{
	for (unsigned int i = 0; i < Poly.GetLineCount(); i++)
	{
		C2DPoint pt = Poly.GetLine(i)->GetPointFrom();
		this->ScaleAndOffSet(pt);

		pDC->SetPixel((int)pt.x, (int)pt.y, colorRef);
	}
}

/**--------------------------------------------------------------------------<BR>
CGeoDraw::DrawPoints
\brief DrawPoints.
<P>---------------------------------------------------------------------------*/
void CGeoDraw::DrawPoints(const C2DHoledPolyBase& Poly, CDC* pDC, unsigned long colorRef) const
{
	const C2DPolyBase* pPoly = Poly.GetRim();

	if (pPoly)
	{
		DrawPoints(*pPoly, pDC, colorRef);
	}

	for (unsigned int i = 0; i < Poly.GetHoleCount(); i++)
	{
		pPoly = Poly.GetHole(i);

		if (pPoly)
		{
			DrawPoints(*pPoly, pDC, colorRef);
		}
	}
}


/**--------------------------------------------------------------------------<BR>
CGeoDraw::Draw
\brief Draw.
<P>---------------------------------------------------------------------------*/
void CGeoDraw::Draw(const C2DLine& Line, CDC* pDC, CPen* pPen) const
{
	CPen* oldpen = 0;

	if (pPen != 0)
		oldpen = pDC->SelectObject(pPen);

	// Offset then scale.

	C2DPoint ptFrom = Line.point - (*m_pOffset);
	ptFrom *= (*m_pScale);
	C2DPoint ptTo = Line.GetPointTo() - (*m_pOffset);
	ptTo *= (*m_pScale);

	pDC->MoveTo( (int) ptFrom.x,(int) ptFrom.y );

	pDC->LineTo( (int) ptTo.x,(int) ptTo.y );

	if (oldpen != 0)
		pDC->SelectObject(oldpen);

}

/**--------------------------------------------------------------------------<BR>
CGeoDraw::Draw
\brief Draw.
<P>---------------------------------------------------------------------------*/
void CGeoDraw::Draw(const C2DArc& Arc, CDC* pDC, CPen* pPen) const
{
	CPen* oldpen = 0;

	if (pPen != 0 )
		oldpen = pDC->SelectObject(pPen);

	C2DCircle Circle(Arc.GetCircleCentre() - (*m_pOffset), Arc.GetRadius());

	C2DRect rect;
	Circle.GetBoundingRect(rect);
	rect.Scale(*m_pScale);

	C2DPoint pt2DFrom = Arc.GetPointFrom();
	pt2DFrom -= (*m_pOffset);
	pt2DFrom *= (*m_pScale);

	C2DPoint pt2DTo = Arc.GetPointTo();

	pt2DTo -= (*m_pOffset);
	pt2DTo *= (*m_pScale);

	CPoint ptFrom( (int) pt2DFrom.x, (int) pt2DFrom.y);
	CPoint ptTo( (int) pt2DTo.x, (int) pt2DTo.y);

	if (ptFrom.x == ptTo.x && ptFrom.y == ptTo.y)
	{
		_ASSERT(0);
	}

	if (Arc.GetArcOnRight() ^ (((*m_pScale).x * (*m_pScale).y) < 0) ) // Reverse for windows
		pDC->SetArcDirection (AD_CLOCKWISE);
	else
		pDC->SetArcDirection (AD_COUNTERCLOCKWISE);

	CRect WinRect( (int)rect.GetTopLeft().x, (int)rect.GetBottomRight().y, (int)rect.GetBottomRight().x, (int)rect.GetTopLeft().y);


	pDC->Arc( &WinRect, ptFrom,  ptTo);

	if (oldpen != 0)
		pDC->SelectObject(oldpen);

}


/**--------------------------------------------------------------------------<BR>
CGeoDraw::Draw
\brief Draw.
<P>---------------------------------------------------------------------------*/
void CGeoDraw::Draw(const C2DCircle& Circle, CDC* pDC, CPen* pPen) const
{
	CPen* oldpen = 0;
	if (pPen != 0) 
		oldpen = pDC->SelectObject(pPen);

	CBrush Brush;
	LOGBRUSH log;
	log.lbStyle = BS_HOLLOW;
	Brush.CreateBrushIndirect(&log);

	CBrush* oldBrush = pDC->SelectObject(&Brush);

	const C2DPoint& ptCen = Circle.GetCentre();
	double dRadius = Circle.GetRadius();
	const C2DPoint& ptOffSet = GetOffSet();
	const C2DPoint& ptScale = GetScale();

	int x1 = (int) (( (ptCen.x - dRadius) - ptOffSet.x ) * ptScale.x);
	int y1 = (int) (( (ptCen.y - dRadius) - ptOffSet.y ) * ptScale.y);
	int x2 = (int) (( (ptCen.x + dRadius) - ptOffSet.x ) * ptScale.x);
	int y2 = (int) (( (ptCen.y + dRadius) - ptOffSet.y ) * ptScale.y);

	pDC->Ellipse(  x1 , y1, x2, y2);

	if (oldpen != 0)
		pDC->SelectObject(oldpen);

	pDC->SelectObject(oldBrush);


}

/**--------------------------------------------------------------------------<BR>
CGeoDraw::Draw
\brief Draw.
<P>---------------------------------------------------------------------------*/
void CGeoDraw::Draw(const C2DPoint& Pt, CDC* pDC, CPen* pPen) const
{
	C2DCircle cir(Pt, 1);

	Draw( cir, pDC, pPen );
}

/**--------------------------------------------------------------------------<BR>
CGeoDraw::Draw
\brief Draw.
<P>---------------------------------------------------------------------------*/
void CGeoDraw::Draw(const C2DRect& Rect, CDC* pDC, CPen* pPen) const
{

	C2DPoint BL = C2DPoint(Rect.GetTopLeft().x, Rect.GetBottomRight().y );
	C2DPoint TR = C2DPoint(Rect.GetBottomRight().x,  Rect.GetTopLeft().y );

	C2DLine Left( BL,  Rect.GetTopLeft());
	Draw(Left, pDC, pPen );

	C2DLine Top(  Rect.GetTopLeft(),TR );
	Draw(Top, pDC, pPen );

	C2DLine Right( TR, Rect.GetBottomRight());
	Draw(Right, pDC, pPen );

	C2DLine Bottom( Rect.GetBottomRight(), BL);
	Draw(Bottom, pDC, pPen );
}

/**--------------------------------------------------------------------------<BR>
CGeoDraw::Draw
\brief Draw.
<P>---------------------------------------------------------------------------*/
void CGeoDraw::Draw(const C2DPolyBase& Poly, CDC* pDC, CPen* pPen) const
{
	const C2DLineBaseSet& Lines = Poly.GetLines();


	if (Lines.size() == 0)
		return;

	CPen* oldpen = 0;
	if (pPen != 0)
		pDC->SelectObject(pPen);

	pDC->AbortPath();

	C2DPoint StartPt =  Lines[0].GetPointFrom() ;
	StartPt = C2DPoint( StartPt - (*m_pOffset)  ) * (*m_pScale); 

	pDC->MoveTo( (int) StartPt.x, (int) StartPt.y);
	pDC->BeginPath();
	for (unsigned int i = 0 ; i < Lines.size(); i++)
	{
		C2DPoint NextPt = Lines[i].GetPointTo();
		NextPt = C2DPoint( NextPt - (*m_pOffset)  ) * (*m_pScale); 

		if (Lines[i].GetType() == C2DBase::StraightLine)
		{
			pDC->LineTo (  (int) NextPt.x, (int) NextPt.y);
		}
		else
		{
			const C2DArc& Arc = dynamic_cast<const C2DArc&> ( *Poly.GetLine(i));
			C2DCircle Circle(Arc.GetCircleCentre() - (*m_pOffset), Arc.GetRadius());

			C2DRect rect;
			Circle.GetBoundingRect(rect);
			rect.Scale(*m_pScale);

			CRect WinRect(  (int)rect.GetLeft(), (int)rect.GetBottom(), 
							(int)rect.GetRight(), (int)rect.GetTop()  );

			C2DPoint PreviousPt =  Arc.GetPointFrom();
			ScaleAndOffSet(PreviousPt);

			CPoint ptFrom( (int) PreviousPt.x, (int) PreviousPt.y);
			CPoint ptTo( (int) NextPt.x, (int) NextPt.y);

			if (ptFrom != ptTo)
			{
				if (  Arc.GetArcOnRight() ^ (((*m_pScale).x * (*m_pScale).y) < 0)  ) // Reverse for windows
					pDC->SetArcDirection (AD_CLOCKWISE);
				else
					pDC->SetArcDirection (AD_COUNTERCLOCKWISE);

				pDC->ArcTo( WinRect, ptFrom,  ptTo);
			}
		}
	}

	pDC->EndPath();
	pDC->StrokePath();

	if (oldpen != 0)
		pDC->SelectObject(oldpen);
}

/**--------------------------------------------------------------------------<BR>
CGeoDraw::Draw
\brief Draw.
<P>---------------------------------------------------------------------------*/
void CGeoDraw::Draw(const C2DSegment& Seg, CDC* pDC, CPen* pPen) const
{
	Draw( Seg.GetArc(), pDC, pPen);
	Draw( Seg.GetLine(), pDC, pPen);
}

/**--------------------------------------------------------------------------<BR>
CGeoDraw::Draw
\brief Draw.
<P>---------------------------------------------------------------------------*/
void CGeoDraw::Draw(const C2DTriangle& Triangle, CDC* pDC, CPen* pPen) const
{
	CPen* oldpen = 0;

	if (pPen != 0)
		oldpen = pDC->SelectObject(pPen);

	//// Offset then scale.
	C2DPoint m_pt1T = C2DPoint((Triangle.GetPoint1() - (*m_pOffset)  ))* (*m_pScale) ;
	C2DPoint m_pt2T = C2DPoint((Triangle.GetPoint2() - (*m_pOffset) ))* (*m_pScale);
	C2DPoint m_pt3T = C2DPoint((Triangle.GetPoint3() - (*m_pOffset) ))* (*m_pScale);

	pDC->MoveTo( (int)m_pt1T.x, (int) m_pt1T.y);
	pDC->LineTo( (int)m_pt2T.x, (int) m_pt2T.y);
	pDC->LineTo( (int)m_pt3T.x, (int) m_pt3T.y);
	pDC->LineTo( (int)m_pt1T.x, (int) m_pt1T.y);

	if (oldpen != 0)
		pDC->SelectObject(oldpen);

}

/**--------------------------------------------------------------------------<BR>
CGeoDraw::Draw
\brief Draw.
<P>---------------------------------------------------------------------------*/
void CGeoDraw::Draw(const C2DHoledPolyBase& HoledPoly, CDC* pDC, CPen* pPen) const
{
	
	if (HoledPoly.GetRim() == 0)
		return;

	Draw(*HoledPoly.GetRim(), pDC, pPen);

	for (unsigned int i = 0 ; i < HoledPoly.GetHoleCount(); i++)
	{
		Draw( *HoledPoly.GetHole(i), pDC, pPen);
	}
}

/**--------------------------------------------------------------------------<BR>
CGeoDraw::DrawFilled
\brief Draw filled.
<P>---------------------------------------------------------------------------*/
void CGeoDraw::DrawFilled(const C2DPolyBase& Poly, CDC* pDC, CBrush* pBrush , CPen* pPen ) const
{
	if (Poly.GetLineCount() == 0 )
		return;

	CPen* oldpen = 0;
	if (pPen)
		oldpen = pDC->SelectObject(pPen);
	CBrush* oldBrush = 0;
	if (pBrush)
		oldBrush = pDC->SelectObject(pBrush);

	pDC->AbortPath();
	C2DPoint StartPt = Poly.GetLine(0)->GetPointFrom(); 
	ScaleAndOffSet(StartPt);
	pDC->MoveTo( (int) StartPt.x, (int) StartPt.y);
	pDC->BeginPath();

	for (unsigned int i = 0 ; i < Poly.GetLineCount(); i++)
	{
		C2DPoint NextPt = Poly.GetLine(i)->GetPointTo();
		ScaleAndOffSet(NextPt);

		if (Poly.GetLine(i)->GetType() == C2DBase::StraightLine)
		{
			pDC->LineTo (  (int) NextPt.x, (int) NextPt.y);
		}
		else if (Poly.GetLine(i)->GetType() == C2DBase::ArcedLine)
		{
			const C2DArc& Arc = dynamic_cast<const C2DArc&> ( *Poly.GetLine(i));
			C2DCircle Circle(Arc.GetCircleCentre() - (*m_pOffset), Arc.GetRadius());

			C2DRect rect;
			Circle.GetBoundingRect(rect);
			rect.Scale(*m_pScale);

			CRect WinRect(  (int)rect.GetLeft(), (int)rect.GetBottom(), 
							(int)rect.GetRight(), (int)rect.GetTop()  );

			C2DPoint PreviousPt =  Arc.GetPointFrom();
			ScaleAndOffSet(PreviousPt);

			CPoint ptFrom( (int) PreviousPt.x, (int) PreviousPt.y);
			CPoint ptTo( (int) NextPt.x, (int) NextPt.y);

			if (ptFrom != ptTo)
			{
				if (  Arc.GetArcOnRight() ^ (((*m_pScale).x * (*m_pScale).y) < 0)  ) // Reverse for windows
					pDC->SetArcDirection (AD_CLOCKWISE);
				else
					pDC->SetArcDirection (AD_COUNTERCLOCKWISE);

				pDC->ArcTo( WinRect, ptFrom,  ptTo);
			}
		}
	}

	pDC->EndPath();
	pDC->StrokeAndFillPath();

	if (oldpen)
		pDC->SelectObject(oldpen);
	if (oldBrush)
		pDC->SelectObject(oldBrush);
}

/**--------------------------------------------------------------------------<BR>
CGeoDraw::DrawFilled
\brief Draw filled.
<P>---------------------------------------------------------------------------*/
void CGeoDraw::DrawFilled(const C2DHoledPolyBase& HoledPoly, CDC* pDC, CBrush* pBrush, CPen* pPen) const
{
	const C2DPolyBase* pRim = HoledPoly.GetRim();

	if ( pRim == 0)
		return;

	if (pRim->GetLineCount() == 0 )
		return;

	CPen* oldpen = 0;
	if (pPen)
		oldpen = pDC->SelectObject(pPen);

	CBrush* oldBrush = 0;
	if (pBrush)
		oldBrush = pDC->SelectObject(pBrush);



	pDC->AbortPath();
	//C2DPoint StartPt = pRim->GetLine(0)->GetPointFrom(); 
	//pDC->MoveTo( (int) StartPt.x, (int) StartPt.y);
	//pDC->BeginPath();

	/// *****************  RIM ***************
//	const C2DPolyBase* Poly = GetRim();

	int nUndrawn = 0;
	int nDrawn = 0;

	C2DPoint StartPt = pRim->GetLine(0)->GetPointFrom(); 

	ScaleAndOffSet(StartPt);

	pDC->MoveTo( (int) StartPt.x, (int) StartPt.y);
	pDC->BeginPath();

	for (unsigned int i = 0 ; i < pRim->GetLineCount(); i++)
	{
		const C2DLineBase* pLine = pRim->GetLine(i);
		C2DPoint NextPt = pLine->GetPointTo();
		ScaleAndOffSet(NextPt);
		C2DPoint PreviousPt =  pLine->GetPointFrom();
		ScaleAndOffSet(PreviousPt);
		CPoint ptFrom( (int) PreviousPt.x, (int) PreviousPt.y);
		CPoint ptTo( (int) NextPt.x, (int) NextPt.y);

		if (ptFrom != ptTo)
		{
			if (pLine->GetType() == C2DBase::StraightLine)
			{
				pDC->LineTo (  ptTo.x, ptTo.y);
			}
			else if (pLine->GetType() == C2DBase::ArcedLine)
			{
				const C2DArc& Arc = dynamic_cast<const C2DArc&> ( *pRim->GetLine(i));
				C2DCircle Circle(Arc.GetCircleCentre() - (*m_pOffset), Arc.GetRadius());

				C2DRect rect;
				Circle.GetBoundingRect(rect);
				rect.Scale(*m_pScale);

				CRect WinRect(  (int)rect.GetLeft(), (int)rect.GetBottom(), 
								(int)rect.GetRight(), (int)rect.GetTop()  );

				if (  Arc.GetArcOnRight() ^ (((*m_pScale).x * (*m_pScale).y) < 0)  ) // Reverse for windows
					pDC->SetArcDirection (AD_CLOCKWISE);
				else
					pDC->SetArcDirection (AD_COUNTERCLOCKWISE);

				pDC->ArcTo( WinRect, ptFrom,  ptTo);
			}
			nDrawn++;
		}
		else
		{
			nUndrawn++;
		}
	}




	/// *****************  HOLES *********************
	for (unsigned int i = 0 ; i < HoledPoly.GetHoleCount(); i++)
	{
		const C2DPolyBase* Poly = HoledPoly.GetHole(i);

		if (Poly->GetLineCount() != 0)
		{
			C2DPoint StartPt = Poly->GetLine(0)->GetPointFrom(); 
			ScaleAndOffSet( StartPt );
			pDC->MoveTo( (int) StartPt.x, (int) StartPt.y);

			for (unsigned int i = 0 ; i < Poly->GetLineCount(); i++)
			{

				const C2DLineBase* pLine = Poly->GetLine(i);
				C2DPoint NextPt = pLine->GetPointTo();
				ScaleAndOffSet( NextPt );
				C2DPoint PreviousPt =  pLine->GetPointFrom();
				ScaleAndOffSet(PreviousPt);
				CPoint ptFrom( (int) PreviousPt.x, (int) PreviousPt.y);
				CPoint ptTo( (int) NextPt.x, (int) NextPt.y);

				if (ptFrom != ptTo)
				{
					if (Poly->GetLine(i)->GetType() == C2DBase::StraightLine)
					{
						pDC->LineTo (  ptTo.x, ptTo.y);
					}
					else if (Poly->GetLine(i)->GetType() == C2DBase::ArcedLine)
					{
						const C2DArc& Arc = dynamic_cast<const C2DArc&> ( *Poly->GetLine(i));
						C2DCircle Circle(Arc.GetCircleCentre() - (*m_pOffset), Arc.GetRadius());

						C2DRect rect;
						Circle.GetBoundingRect(rect);
						rect.Scale(*m_pScale);
						CRect WinRect(  (int)rect.GetLeft(), (int)rect.GetBottom(), 
										(int)rect.GetRight(), (int)rect.GetTop()  );

						C2DPoint PreviousPt =  Arc.GetPointFrom();
						ScaleAndOffSet( PreviousPt );


						CPoint ptFrom( (int) PreviousPt.x, (int) PreviousPt.y);
						CPoint ptTo( (int) NextPt.x, (int) NextPt.y);


						if (  Arc.GetArcOnRight()  ^ (((*m_pScale).x * (*m_pScale).y) < 0) ) // Reverse for windows
							pDC->SetArcDirection (AD_CLOCKWISE);
						else
							pDC->SetArcDirection (AD_COUNTERCLOCKWISE);

						pDC->ArcTo( WinRect, ptFrom,  ptTo);
						
					}
				}
			}
		}
	}

	pDC->EndPath();

	pDC->StrokeAndFillPath();

	if (oldpen)
		pDC->SelectObject(oldpen);
	if (oldBrush)
		pDC->SelectObject(oldBrush);

}

/**--------------------------------------------------------------------------<BR>
CGeoDraw::DrawFilled
\brief Draw filled.
<P>---------------------------------------------------------------------------*/
void CGeoDraw::DrawFilled(const C2DCircle& Circle, CDC* pDC, CBrush* pBrush,  CPen* pPen)  const
{
	CPen* oldpen = 0;
	if (pPen != 0) 
		oldpen = pDC->SelectObject(pPen);
	CBrush* oldBrush = 0;
	if (pBrush)
		CBrush* oldBrush = pDC->SelectObject(pBrush);

	const C2DPoint& ptCen = Circle.GetCentre();
	double dRadius = Circle.GetRadius();
	const C2DPoint& ptOffSet = GetOffSet();
	const C2DPoint& ptScale = GetScale();

	int x1 = (int) (( (ptCen.x - dRadius) - ptOffSet.x ) * ptScale.x);
	int y1 = (int) (( (ptCen.y - dRadius) - ptOffSet.y ) * ptScale.y);
	int x2 = (int) (( (ptCen.x + dRadius) - ptOffSet.x ) * ptScale.x);
	int y2 = (int) (( (ptCen.y + dRadius) - ptOffSet.y ) * ptScale.y);

	pDC->Ellipse(  x1 , y1, x2, y2);

	if (oldpen != 0)
		pDC->SelectObject(oldpen);
	if (oldBrush)
		pDC->SelectObject(oldBrush);

}


/**--------------------------------------------------------------------------<BR>
CGeoDraw::DrawFilled
\brief Draw filled.
<P>---------------------------------------------------------------------------*/
void CGeoDraw::DrawFilled(const C2DRect& Rect, CDC* pDC, CBrush* pBrush , CPen* pPen) const
{
	CPen* oldpen = 0;
	if (pPen != 0) 
		oldpen = pDC->SelectObject(pPen);
	CBrush* oldBrush = 0;
	if (pBrush)
		CBrush* oldBrush = pDC->SelectObject(pBrush);

	C2DPoint ptTL = Rect.GetTopLeft();
	ptTL -= (*m_pOffset);
	ptTL *= (*m_pScale);

	C2DPoint ptBR = Rect.GetBottomRight();
	ptBR -= (*m_pOffset);
	ptBR *= (*m_pScale);

	pDC->Rectangle( (int)ptTL.x, (int)ptTL.y, (int)ptBR.x, (int)ptBR.y);

	if (oldpen != 0)
		pDC->SelectObject(oldpen);
	if (oldBrush)
		pDC->SelectObject(oldBrush);
}


/**--------------------------------------------------------------------------<BR>
CGeoDraw::DrawFilled
\brief Draw filled.
<P>---------------------------------------------------------------------------*/
void CGeoDraw::DrawFilled(const C2DSegment& Seg, CDC* pDC, CBrush* pBrush , CPen* pPen ) const
{
	CPen* oldpen = 0;
	if (pPen)
		oldpen = pDC->SelectObject(pPen);
	CBrush* oldBrush = 0;
	if (pBrush)
		oldBrush = pDC->SelectObject(pBrush);

	pDC->AbortPath();
	C2DPoint StartPt = Seg.GetLine().GetPointFrom();
	ScaleAndOffSet(StartPt);
	pDC->MoveTo( (int) StartPt.x, (int) StartPt.y);
	pDC->BeginPath();


	C2DPoint NextPt = Seg.GetLine().GetPointTo();
	ScaleAndOffSet(NextPt);
	pDC->LineTo (  (int) NextPt.x, (int) NextPt.y);

	C2DCircle Circle(Seg.GetCircleCentre() - (*m_pOffset), Seg.GetArc().GetRadius() );
	C2DRect rect;
	Circle.GetBoundingRect(rect);
	rect.Scale(*m_pScale);

	CRect WinRect(  (int)rect.GetLeft(), (int)rect.GetBottom(), 
					(int)rect.GetRight(), (int)rect.GetTop()  );


	CPoint ptFrom( (int) NextPt.x, (int) NextPt.y);
	CPoint ptTo( (int) StartPt.x, (int) StartPt.y);

	if (ptFrom != ptTo)
	{
		if (  Seg.GetArc().GetArcOnRight() ^ (((*m_pScale).x * (*m_pScale).y) < 0)  ) // Reverse for windows
			pDC->SetArcDirection (AD_CLOCKWISE);
		else
			pDC->SetArcDirection (AD_COUNTERCLOCKWISE);

		pDC->ArcTo( WinRect, ptFrom,  ptTo);
	}


	pDC->EndPath();
	pDC->StrokeAndFillPath();

	if (oldpen)
		pDC->SelectObject(oldpen);
	if (oldBrush)
		pDC->SelectObject(oldBrush);
}

/**--------------------------------------------------------------------------<BR>
CGeoDraw::DrawFilled
\brief Draw filled.
<P>---------------------------------------------------------------------------*/
void CGeoDraw::DrawFilled(const C2DTriangle& Triangle, CDC* pDC, CBrush* pBrush , CPen* pPen ) const
{
	CPen* oldpen = 0;
	if (pPen)
		oldpen = pDC->SelectObject(pPen);
	CBrush* oldBrush = 0;
	if (pBrush)
		oldBrush = pDC->SelectObject(pBrush);

	pDC->AbortPath();
	C2DPoint Pt = Triangle.GetPoint1();
	ScaleAndOffSet(Pt);
	pDC->MoveTo( (int) Pt.x, (int) Pt.y);
	pDC->BeginPath();


	Pt = Triangle.GetPoint2();
	ScaleAndOffSet(Pt);
	pDC->LineTo (  (int) Pt.x, (int) Pt.y);

	Pt = Triangle.GetPoint3();
	ScaleAndOffSet(Pt);
	pDC->LineTo (  (int) Pt.x, (int) Pt.y);

	pDC->EndPath();
	pDC->StrokeAndFillPath();

	if (oldpen)
		pDC->SelectObject(oldpen);
	if (oldBrush)
		pDC->SelectObject(oldBrush);
}



/**--------------------------------------------------------------------------<BR>
CGeoDraw::ScaleAndOffSet
\brief
<P>---------------------------------------------------------------------------*/
void CGeoDraw::ScaleAndOffSet(C2DPoint& pt) const
{
	pt -= (*m_pOffset);
	pt *= (*m_pScale);
}



