/*---------------------------------------------------------------------------
Copyright (C) GeoLib.
This code is used under license from GeoLib (www.geolib.co.uk). This or
any modified versions of this cannot be resold to any other party.
---------------------------------------------------------------------------*/

/**--------------------------------------------------------------------------<BR>
\file GeoView.h
\brief Declaration file for the CGeoView Class.

Declaration file for CGeoView, a class which is used to provide a convenient way
to draw shapes.

\class CGeoView
\brief CGeoView, a class which is used to provide a convenient way
to draw shapes.

This class basically takes a copy of your CDC and draws shapes onto that. Once
the shapes are drawn they can then be copied over onto the main CDC or any other.

The view defaults to view all the shapes.

E.G.

void OnDraw(CDC* pDC)
{

	shapeFile.Load("My polygon file", m_Polygons);
.
.
.
	m_GeoView.Initialise(pDC, const_nViewWidth, const_nViewHeight);

	m_GeoView.CreateDefaultView(m_Polygons);

	m_GeoView.Draw(m_Polygons);

	// Note that once all this is done, only the following needs to be done for a 
	// redraw, unless the shapes change.

	m_GeoView.Display(pDC, m_nXOffset, m_nYOffset);
}
<P>---------------------------------------------------------------------------*/

#pragma once

#include "GeoDraw.h"
#include "ScreenManager.h"
#include "GeoLib.h"
#include <vector>

#ifdef _VIEW_EXPORTING
	#define VIEW_DECLSPEC		__declspec(dllexport)
#else 
	#ifdef _STATIC
		#define VIEW_DECLSPEC
	#else
		#define VIEW_DECLSPEC		__declspec(dllimport)
	#endif
#endif


class CProgressUpdater;
class C2DHoledPolygonSet;
class C2DHoledPolyBaseSet;

class VIEW_DECLSPEC CGeoView
{
public:
	CGeoView(void);

	~CGeoView(void);

	// Sets the screenmanager to look at the whole projection by finding the bounding rect.
	void CreateDefaultView(const C2DHoledPolyBaseSet& polys);
	// True if its been initialised.
	bool IsInitialised(void) const;
	// Initialise.
	void Initialise(CDC* pDC, unsigned int nWidth, unsigned int nHeight);
	// Draw the shapes onto the locally copied CDC only.
	void Draw(const C2DHoledPolyBaseSet& polys);	
	// Access to the local display.
	CDC* GetDisplay(void);
	// Access to the local display.
	const CDC* GetDisplay(void) const;
	// Display the locally drawn shape CDC / bitmap onto the main CDC for viewing.
	void Display(CDC* pDC, unsigned int nXOffset = 0, unsigned int nYOffset = 0);
	// Display the shapes.
	void Display(CDC* pDC, const CRect& rect);
	// Display the shapes.
	void Display(CDC* pDC, unsigned int nXOffset, unsigned int nYOffset, 
								unsigned int nWidth, unsigned int nHeight);
	// Access to the screen manager.
	CScreenManager& GetScreenManager(void);
	// Colour selection.
	void SetBackground(COLORREF rgb);
	// Colour selection.
	void SetBoundaries(COLORREF rgb);
	// Colour selection.
	void SetShapes(COLORREF rgb);
	// Opt to draw only points.
	void SetDrawPointsOnly(bool bPointsonly);
	// Colour selection.
	COLORREF GetBackground(void);
	// Colour selection.
	COLORREF GetBoundaries(void);
	// Colour selection.
	COLORREF GetShapes(void);
	// Assignment.
	void operator=(CGeoView& rOther);

private:

	void ClearCanvas(void);

	void DrawPolygon(const C2DHoledPolyBase& Poly, CDC* pDC, CBrush* pBrush, CPen* pPen);

	void DrawPolygonSet(const C2DHoledPolyBaseSet* Poly, CDC* pDC, CBrush* pBrush, CPen* pPen);

	bool IsDrawable(const C2DPolyBase& Poly) const;



private:

	CScreenManager m_ScreenManager;

	CGeoDraw m_Drawer;

	CBitmap* m_pBitMap;

	CDC* m_pDC;

	COLORREF m_BackGround;

	COLORREF m_Boundaries;
	
	COLORREF m_Shapes;

	bool m_bPointsOnly;
};
