/*---------------------------------------------------------------------------
Copyright (C) GeoLib.
This code is used under license from GeoLib (www.geolib.co.uk). This or
any modified versions of this cannot be resold to any other party.
---------------------------------------------------------------------------*/


/**--------------------------------------------------------------------------<BR>
\file ScreenManager.cpp
\brief Implementation file for the CScreenManager Class.

Implementation file for CScreenManager, a class which can be used to help 
calculate drawing offsets and scales. 
<P>---------------------------------------------------------------------------*/

#include "StdAfx.h"

#include "ScreenManager.h"
#include "C2DVector.h"




/**--------------------------------------------------------------------------<BR>
CScreenManager::CScreenManager
\brief Constructor, note the Y flip defaults to true for windows.
<P>---------------------------------------------------------------------------*/
CScreenManager::CScreenManager(void) : 
m_WindowsRect(0,1,1,0), m_RealRect(0,1,1,0), m_ptOffset(0,0), m_ptScale(0,0)
{
	m_bFlipY = true;
	m_bFlipX = false;
}

/**--------------------------------------------------------------------------<BR>
CScreenManager::CScreenManager
\brief Destructor
<P>---------------------------------------------------------------------------*/
CScreenManager::~CScreenManager(void)
{
}

void CScreenManager::operator=(CScreenManager& rOther)
{
	m_bFlipX = rOther.m_bFlipX;
	m_bFlipY = rOther.m_bFlipY;
	m_ptOffset = rOther.m_ptOffset;
	m_ptScale = rOther.m_ptScale;
	m_RealRect = rOther.m_RealRect;
	m_RealWindowsRect = rOther.m_RealWindowsRect;
	m_WindowsRect = rOther.m_WindowsRect;
}



/**--------------------------------------------------------------------------<BR>
CScreenManager::GetOffset
\brief Gets the alignment setting.
<P>---------------------------------------------------------------------------*/
const C2DPoint& CScreenManager::GetOffset(void) const
{
	return m_ptOffset;
}


/**--------------------------------------------------------------------------<BR>
CScreenManager::GetScale
\brief Gets the scale.
<P>---------------------------------------------------------------------------*/
const C2DPoint& CScreenManager::GetScale(void) const
{
	return m_ptScale;
}


/**--------------------------------------------------------------------------<BR>
CScreenManager::GetScale
\brief Sets the windows rect.
<P>---------------------------------------------------------------------------*/
void CScreenManager::SetWindowsRect(const CRect& WRect)
{
	m_WindowsRect = WRect;

	Calculate();
}

/**--------------------------------------------------------------------------<BR>
CScreenManager::GetScale
\brief Sets the real world rect.
<P>---------------------------------------------------------------------------*/
void CScreenManager::SetRealRect(const C2DRect& RRect)
{
	m_RealRect = RRect;

	Calculate();
}


/**--------------------------------------------------------------------------<BR>
CScreenManager::ZoomOut
\brief This simply grows the real world rectangle.
<P>---------------------------------------------------------------------------*/
void CScreenManager::ZoomOut(double dFactor)
{
	m_RealRect.Grow(dFactor);

	Calculate();
}


/**--------------------------------------------------------------------------<BR>
CScreenManager::ScrollReal
\brief This simply moves the real world rectangle.
<P>---------------------------------------------------------------------------*/
void CScreenManager::ScrollReal(const C2DVector& Vector)
{
	m_RealRect.Move(Vector);

	Calculate();
}


/**--------------------------------------------------------------------------<BR>
CScreenManager::ScrollScreen
\brief This simply moves the real world rectangle but by a scaled, screen based
value e.g. 10 pixels to the right. The image is shown in the same place on
the screen but the image scrolls.
<P>---------------------------------------------------------------------------*/
void CScreenManager::ScrollScreen(const C2DVector& Vector)
{
	m_RealRect.Move(C2DVector( Vector.i / m_ptScale.x, Vector.j / m_ptScale.y) );

	Calculate();
}


/**--------------------------------------------------------------------------<BR>
CScreenManager::GetWindowsRect
\brief Returns the windows rect.
<P>---------------------------------------------------------------------------*/
const CRect& CScreenManager::GetWindowsRect(void) const
{
	return m_WindowsRect;
}


/**--------------------------------------------------------------------------<BR>
CScreenManager::GetRealRect
\brief Returns the real world rect.
<P>---------------------------------------------------------------------------*/
const C2DRect& CScreenManager::GetRealRect(void) const
{
	return m_RealRect;
}

/**--------------------------------------------------------------------------<BR>
CScreenManager::GetRealWindowsRect
\brief Returns the real world rect that the screen rect represents. This is not 
the same as the real world rectangle because the screen rect may have a different
aspect ratio.
<P>---------------------------------------------------------------------------*/
const C2DRect& CScreenManager::GetRealWindowsRect(void) const
{
	return m_RealWindowsRect;
}

/**--------------------------------------------------------------------------<BR>
CScreenManager::Calculate
\brief Calculates all the required offset and scale. The real world rect will 
always all be shown but may have to occupy a sub section of the computer
screen.
<P>---------------------------------------------------------------------------*/
void CScreenManager::Calculate(void)
{
	m_RealWindowsRect = m_RealRect;

	float fWinRatio = (float)m_WindowsRect.Width() / (float)m_WindowsRect.Height();
	float fRealRatio = (float)m_RealRect.Width() / (float)m_RealRect.Height();
	bool bFillX = fRealRatio >= fWinRatio;
	if (bFillX)
	{
		m_ptScale.x = (float)m_WindowsRect.Width() / (float)m_RealRect.Width();
		m_ptScale.y = m_ptScale.x;
		m_RealWindowsRect.GrowHeight(fRealRatio / fWinRatio);
	}
	else
	{
		m_ptScale.y = (float)m_WindowsRect.Height() / (float)m_RealRect.Height();
		m_ptScale.x = m_ptScale.y;
		m_RealWindowsRect.GrowWidth(fWinRatio / fRealRatio );
	}

	if (m_bFlipX) m_ptScale.x = - m_ptScale.x;
	if (m_bFlipY) m_ptScale.y = - m_ptScale.y;

	// Now find 2 points to map onto each other and the rest is easy.
	C2DPoint ptRealCen = m_RealRect.GetCentre();
	CPoint ptWinCen = m_WindowsRect.CenterPoint();

	m_ptOffset.x = ptRealCen.x - (float)ptWinCen.x / m_ptScale.x;
	m_ptOffset.y = ptRealCen.y - (float)ptWinCen.y / m_ptScale.y;
}


/**--------------------------------------------------------------------------<BR>
CScreenManager::MapScreenToReal
\brief Returns a real world point from a point on the screen.
<P>---------------------------------------------------------------------------*/
C2DPoint CScreenManager::MapScreenToReal(const CPoint& pt) const
{

	return (C2DPoint(  m_ptOffset.x + (float)pt.x  / m_ptScale.x,
					   m_ptOffset.y + (float)pt.y / m_ptScale.y));

}


/**--------------------------------------------------------------------------<BR>
CScreenManager::MapRealToScreen
\brief Returns a point on the screen from a real world point.
<P>---------------------------------------------------------------------------*/
CPoint CScreenManager::MapRealToScreen(const C2DPoint& pt) const
{
	return (CPoint(   (int)((pt.x - m_ptOffset.x) * m_ptScale.x),
				      (int)((pt.y - m_ptOffset.y) * m_ptScale.y)   ));

}
