/*---------------------------------------------------------------------------
Copyright (C) GeoLib.
This code is used under license from GeoLib (www.geolib.co.uk). This or
any modified versions of this cannot be resold to any other party.
---------------------------------------------------------------------------*/


/**--------------------------------------------------------------------------<BR>
\file ShapeFile.cpp
\brief Implementation file for the CShapeFile Class.

Implementation file for CShapeFile, a static class for opening/saving to Shapefile
format.
<P>---------------------------------------------------------------------------*/

#include "ShapeFile.h"
#include "Utilities.h"
#include "ProgressUpdater.h"
#include "GeoLib.h"
#include "stdafx.h"

#include <stdio.h>
#include <assert.h>




const int conShapeFileVersion = 1000;
const unsigned int conShapeFileCode = 9994; // Hex code is 0000270A;
const short conUnusedBytes = 20; // Unused bytes;
const int conSFPolygon = 5;
const unsigned char INTEGER_BYTES = 4;
const unsigned char POINT_BYTES = 16;
const unsigned char POLY_BB_BYTES = 32;
const unsigned int const_nMinPolySize = 24;


/**--------------------------------------------------------------------------<BR>
\struct sSFBoundingBox
\brief 
<P>---------------------------------------------------------------------------*/
struct sSFBoundingBox
{
	double Xmin;
	double Ymin;
	double XMax;
	double YMax;
	double Zmin;
	double ZMax;
	double MMin;
	double MMax;
};

/**--------------------------------------------------------------------------<BR>
\struct sSFHeader
\brief 
<P>---------------------------------------------------------------------------*/
struct sSFHeader
{
	unsigned int nRecordNumber;
	unsigned int nContentLength;
};

/**--------------------------------------------------------------------------<BR>
\struct sSFPoint
\brief 
<P>---------------------------------------------------------------------------*/
struct sSFPoint
{
	double dX;
	double dY;
};

/**--------------------------------------------------------------------------<BR>
\class CShapeFilePolygon
\brief 
<P>---------------------------------------------------------------------------*/
class CShapeFilePolygon
{
public:
	CShapeFilePolygon(unsigned intnNumParts, unsigned intnNumPoints);

	CShapeFilePolygon();

	~CShapeFilePolygon();

	void Create();

	unsigned int GetRecordLength() const;

	double m_dBox[4]; // XMin, YMin, XMax, YMax;

	unsigned int m_nNumParts;

	unsigned int m_nNumPoints;

	unsigned int* m_pParts;

	sSFPoint* m_pPoints;
};

/**--------------------------------------------------------------------------<BR>
CShapeFilePolygon::CShapeFilePolygon
\brief 
<P>---------------------------------------------------------------------------*/
CShapeFilePolygon::CShapeFilePolygon()
{
	memset(m_dBox, 0, sizeof(double)* 4);

	m_nNumParts = 0;
	m_nNumPoints = 0;

	m_pParts = NULL;
	m_pPoints = NULL;
}


/**--------------------------------------------------------------------------<BR>
CShapeFilePolygon::Create
\brief 
<P>---------------------------------------------------------------------------*/
void CShapeFilePolygon::Create()
{
	m_pParts = new unsigned int[m_nNumParts];
	m_pPoints = new sSFPoint[m_nNumPoints];
}


/**--------------------------------------------------------------------------<BR>
CShapeFilePolygon::CShapeFilePolygon
\brief 
<P>---------------------------------------------------------------------------*/
CShapeFilePolygon::CShapeFilePolygon(unsigned int nNumParts, unsigned int nNumPoints)
{
	m_nNumParts = nNumParts;
	m_nNumPoints = nNumPoints;

	m_pParts = new unsigned int[nNumParts];
	m_pPoints = new sSFPoint[nNumPoints];
}


/**--------------------------------------------------------------------------<BR>
CShapeFilePolygon::~CShapeFilePolygon
\brief 
<P>---------------------------------------------------------------------------*/
CShapeFilePolygon::~CShapeFilePolygon()
{
	delete[] m_pParts;
	delete[] m_pPoints;
}


/**--------------------------------------------------------------------------<BR>
CShapeFilePolygon::GetRecordLength
\brief 
<P>---------------------------------------------------------------------------*/
unsigned int CShapeFilePolygon::GetRecordLength() const
{
	return const_nMinPolySize + m_nNumParts * INTEGER_BYTES + 
			m_nNumPoints * POINT_BYTES;
}



/**--------------------------------------------------------------------------<BR>
CShapeFile::
\brief 
<P>---------------------------------------------------------------------------*/
bool CShapeFile::OpenFile(FILE* pFile)
{
	// Report failure if file cannot be opened.
//	if (!fileToLoad.Open (strFileName, CFile::modeRead))
//		return false;
	// ***********  FILE CODE  **********
	unsigned int nFileCode = 0; // first 4 bytes are the file code (int = 4 bytes)

	if (fread(&nFileCode, sizeof(nFileCode), 1, pFile) != 1)
		return false;

	Utilities::ReverseByteOrder(nFileCode);

	if (nFileCode != conShapeFileCode)
		return false;
	// ***********  UNUSED BYTES  **********
	__int8 nUnusedBytes[conUnusedBytes]; // 20 bytes unused.

	if (fread(&nUnusedBytes, conUnusedBytes, 1 , pFile) != 1)
		return false;


	// ***********  LENGTH  **********
	int nLength;
	if (fread(&nLength, sizeof(int), 1, pFile) != 1)
		return false;

	Utilities::ReverseByteOrder(&nLength, sizeof(nLength));

//	ASSERT (nLength == fileToLoad.GetLength()/2);


	// No need to reverse the rest
	// ***********  VERSION CODE  **********
	int nVersion;
	if (fread(&nVersion, sizeof(int), 1, pFile) != 1)	
		return false;

	// ***********       TYPE    **********
	int nShapeType;
	if (fread(&nShapeType, sizeof(int), 1, pFile) != 1)
		return false;

	// ***********       BOUNDING BOX    **********
	sSFBoundingBox sBoundingBox;
	if (fread(&sBoundingBox, sizeof(sSFBoundingBox), 1, pFile) != 1)
		return false;

	return nShapeType == conSFPolygon;
}


/**--------------------------------------------------------------------------<BR>
CShapeFile::
\brief 
<P>---------------------------------------------------------------------------*/
bool CShapeFile::Load(const char* strFileName, vector<C2DHoledPolygonSet*>& aPolygonSets, 
																CProgressUpdater* pUpdater)
{
	FILE* pFile = fopen(strFileName, "rb");

	bool bResult = false;

	if (pFile)
	{
		fseek(pFile, 0, SEEK_END);
		long int nLength = ftell(pFile);
		fseek(pFile, 0, SEEK_SET);

		if (OpenFile(pFile))
		{
			bResult = LoadPolygonFile(pFile, aPolygonSets, nLength, pUpdater);
		}
		fclose(pFile);
	}

	return bResult;
}


/**--------------------------------------------------------------------------<BR>
CShapeFile::
\brief 
<P>---------------------------------------------------------------------------*/
bool CShapeFile::Load(const char* strFileName, C2DHoledPolygonSet& aPolygons, 
																CProgressUpdater* pUpdater)
{
	FILE* pFile = fopen(strFileName, "rb");

	bool bResult = false;

	if (pFile)
	{
		fseek(pFile, 0, SEEK_END);
		long int nLength = ftell(pFile);
		fseek(pFile, 0, SEEK_SET);

		if (OpenFile(pFile))
		{
			bResult = LoadPolygonFile(pFile, aPolygons, nLength, pUpdater);
		}
		fclose(pFile);
	}

	return bResult;
}



/**--------------------------------------------------------------------------<BR>
CShapeFile::
\brief 
<P>---------------------------------------------------------------------------*/
bool CShapeFile::Load(const char* strFileName, C2DHoledPolyBaseSet& aPolygons, 
																CProgressUpdater* pUpdater)
{
	FILE* pFile = fopen(strFileName, "rb");

	bool bResult = false;

	if (pFile)
	{
		fseek(pFile, 0, SEEK_END);
		long int nLength = ftell(pFile);
		fseek(pFile, 0, SEEK_SET);

		if (OpenFile(pFile))
		{
			bResult = LoadPolygonFile(pFile, aPolygons, nLength, pUpdater);
		}
		fclose(pFile);
	}

	return bResult;
}


/**--------------------------------------------------------------------------<BR>
CShapeFile::
\brief 
<P>---------------------------------------------------------------------------*/
bool CShapeFile::LoadPolygonFile(FILE* pFile, C2DHoledPolygonSet& aPolygons, 
										long int nLength, CProgressUpdater* pUpdater)
{
	while (ftell(pFile) < nLength)
	{
		CShapeFilePolygon cPoly;

		if (ReadPolygon(pFile, cPoly))
		{
			C2DHoledPolygonSet* pPolySet = CreatePolygonSet(&cPoly);

			if (pPolySet)
			{
				aPolygons << *pPolySet;
				delete pPolySet;
			}
		}
		if (pUpdater)
			pUpdater->UpdateProgress(unsigned int (ftell(pFile) * 100.0 / nLength));

	}
	return true;
}


/**--------------------------------------------------------------------------<BR>
CShapeFile::
\brief 
<P>---------------------------------------------------------------------------*/
bool CShapeFile::LoadPolygonFile(FILE* pFile, C2DHoledPolyBaseSet& aPolygons, 
							long int nLength, CProgressUpdater* pUpdater)
{
	while (ftell(pFile) < nLength)
	{
		CShapeFilePolygon cPoly;

		if (ReadPolygon(pFile, cPoly))
		{
			C2DHoledPolygonSet* pPolySet = CreatePolygonSet(&cPoly);

			if (pPolySet)
			{
				aPolygons << *pPolySet;
				delete pPolySet;
			}
		}
		if (pUpdater)
			pUpdater->UpdateProgress(unsigned int (ftell(pFile) * 100.0 / nLength));

	}
	return true;



}

/**--------------------------------------------------------------------------<BR>
CShapeFile::
\brief 
<P>---------------------------------------------------------------------------*/
bool CShapeFile::LoadPolygonFile(FILE* pFile, vector<C2DHoledPolygonSet*>& aPolygonSets, 
										long int nLength, CProgressUpdater* pUpdater)
{
	while (ftell(pFile) < nLength)
	{
		CShapeFilePolygon cPoly;

		if (ReadPolygon(pFile, cPoly))
		{
			C2DHoledPolygonSet* pPolySet = CreatePolygonSet(&cPoly);

			if (pPolySet)
				aPolygonSets.push_back(pPolySet);
		}
		if (pUpdater)
			pUpdater->UpdateProgress(unsigned int (ftell(pFile) * 100.0 / nLength));

	}
	return true;
}




/**--------------------------------------------------------------------------<BR>
CShapeFile::
\brief 
<P>---------------------------------------------------------------------------*/
bool CShapeFile::ReadPolygon(FILE* pFile, CShapeFilePolygon& cPoly)
{
	sSFHeader sfhead;

	if (fread(&sfhead.nRecordNumber, INTEGER_BYTES, 1, pFile) != 1)
		return false;

	Utilities::ReverseByteOrder(sfhead.nRecordNumber);

	if (fread(&sfhead.nContentLength, INTEGER_BYTES, 1, pFile) != 1)
		return false;

	Utilities::ReverseByteOrder(sfhead.nContentLength);

	// Now Read the shape
	int nShapeType;
	if (fread(&nShapeType, INTEGER_BYTES, 1, pFile) != 1)
		return false;

	if (nShapeType != conSFPolygon)
		return false;

	if (fread(&cPoly.m_dBox, POLY_BB_BYTES, 1, pFile) != 1)
		return false;

	if (fread(&cPoly.m_nNumParts, INTEGER_BYTES, 1, pFile) != 1)
		return false;

	if (fread(&cPoly.m_nNumPoints, INTEGER_BYTES, 1, pFile) != 1)
		return false;

	cPoly.Create();

	if (fread(cPoly.m_pParts, cPoly.m_nNumParts * INTEGER_BYTES, 1, pFile) != 1)
		return false;

	if (fread(cPoly.m_pPoints, cPoly.m_nNumPoints * sizeof(sSFPoint), 1, pFile) != 1)
		return false;


	return true;
}





/**--------------------------------------------------------------------------<BR>
CShapeFile::
\brief 
<P>---------------------------------------------------------------------------*/
C2DHoledPolygonSet* CShapeFile::CreatePolygonSet(const CShapeFilePolygon* pPoly)
{
	// The result to return.
	C2DHoledPolygonSet* paGeoHoledPolys = new C2DHoledPolygonSet();

	C2DHoledPolygon* pGeoHoledPoly = NULL;

	// Go through all the parts creating a poly from them
	for (unsigned int p = 0; p < pPoly->m_nNumParts; p++)
	{
		unsigned int nStartIndex = pPoly->m_pParts[p];

		unsigned int nCount;
			
		if (p + 1 < pPoly->m_nNumParts)
			nCount = pPoly->m_pParts[p + 1] - nStartIndex; // End is 1 less than the start of the next
		else
			nCount = pPoly->m_nNumPoints - nStartIndex; // End is the last point.

		// Create the simple polygon - may be clockwise or not depending on whether a hole.
		C2DPolygon* pNewPoly = CreatePolygon((double*)&pPoly->m_pPoints[nStartIndex], nCount);

		// If the holed poly has not been created then make one and add the rim.
		if (pGeoHoledPoly == NULL)
		{
			pGeoHoledPoly = new C2DHoledPolygon;
			// Add the new polygon
			paGeoHoledPolys->Add(pGeoHoledPoly);
			// It should be clockwise.
			assert(pNewPoly->IsClockwise());

			pGeoHoledPoly->SetRimDirect(pNewPoly);
		}
		else if (pNewPoly->IsClockwise())
		{
			// This is a new rim so add it.
			pGeoHoledPoly = new C2DHoledPolygon;

			paGeoHoledPolys->Add(pGeoHoledPoly);

			pGeoHoledPoly->SetRimDirect(pNewPoly);
		}
		else
		{
			// This is a hole. Geolib likes it clockwise, ESRI likes it anticlockwise.
			// NOTE that ESRI does not say whether this hole is that of the preceeding rim but assume it is.
			// Verify this in debug.
			pNewPoly->ReverseDirection();

		///	ASSERT(pGeoHoledPoly->GetRim()->Contains(*pNewPoly));

			unsigned int i = 0;
			bool bFound = false;
			unsigned int nCount = paGeoHoledPolys->size();
			while (i <  nCount && !bFound)
			{
				C2DHoledPolygon* pTestPoly = paGeoHoledPolys->GetAt(i);

				if (pTestPoly->GetRim()->Contains(*pNewPoly))
				{
					pTestPoly->AddHoleDirect(pNewPoly);
					bFound = true;
				}
				i++;
			}

			if (!bFound)
				pGeoHoledPoly->AddHoleDirect(pNewPoly);
		}
	}

	return paGeoHoledPolys;
}




/**--------------------------------------------------------------------------<BR>
CShapeFile::
\brief 
<P>---------------------------------------------------------------------------*/
C2DPolygon* CShapeFile::CreatePolygon(const double* pPointList, unsigned int nPointCount)
{
	C2DPolygon* pPoly = new C2DPolygon();
	
	pPoly->Create(pPointList, nPointCount);

	return pPoly;
}





/**--------------------------------------------------------------------------<BR>
CShapeFile::
\brief 
<P>---------------------------------------------------------------------------*/
bool CShapeFile::Save(const char* strFileName, const vector<C2DHoledPolygonSet*>& aPolygonSets, 
																CProgressUpdater* pUpdater)
{
	bool bResult = false;

	FILE* pFile = fopen(strFileName, "wb");

	if (pFile)
	{
		if (OpenSave(pFile))
		{
			WriteBoundingBox(pFile, aPolygonSets);

			for (unsigned int i = 0; i < aPolygonSets.size(); i++)
			{
				CShapeFilePolygon rSFPoly;

				CreatePolygon(*aPolygonSets[i], rSFPoly);
			
				Write(&rSFPoly, i, pFile);
			}

			WriteFileLength(pFile);

			bResult = true;
		}
		fclose(pFile);
	}
	return bResult;
}


/**--------------------------------------------------------------------------<BR>
CShapeFile::
\brief 
<P>---------------------------------------------------------------------------*/
bool CShapeFile::Save(const char* strFileName, const C2DHoledPolygonSet& aPolygons, 
																CProgressUpdater* pUpdater)
{
	bool bResult = false;

	FILE* pFile = fopen(strFileName, "wb");

	if (pFile)
	{
		if (OpenSave(pFile))
		{
			WriteBoundingBox(pFile, aPolygons);

			for (unsigned int i = 0; i < aPolygons.size(); i++)
			{
				CShapeFilePolygon rSFPoly;

				CreatePolygon(aPolygons[i], rSFPoly);
			
				Write(&rSFPoly, i, pFile);
			}

			WriteFileLength(pFile);

			fclose(pFile);

			bResult = true;
		}
	}

	return bResult;
}


/**--------------------------------------------------------------------------<BR>
CShapeFile::
\brief 
<P>---------------------------------------------------------------------------*/
bool CShapeFile::Save(const char* strFileName, const C2DHoledPolyBaseSet& aPolygons, 
																CProgressUpdater* pUpdater)
{
	bool bResult = false;

	FILE* pFile = fopen(strFileName, "wb");

	if (pFile)
	{
		if (OpenSave(pFile))
		{

			WriteBoundingBox(pFile, aPolygons);

			for (unsigned int i = 0; i < aPolygons.size(); i++)
			{
				CShapeFilePolygon rSFPoly;

				CreatePolygon(aPolygons[i], rSFPoly);
			
				Write(&rSFPoly, i, pFile);
			}

			WriteFileLength(pFile);

			fclose(pFile);

			bResult = true;
		}
	}

	return bResult;
}


/**--------------------------------------------------------------------------<BR>
CShapeFile::
\brief 
<P>---------------------------------------------------------------------------*/
bool CShapeFile::OpenSave(FILE* pFile)
{
	// ***********  FILE CODE  **********
	unsigned int nFileCode = conShapeFileCode; // first 4 bytes are the file code (int = 4 bytes)

	Utilities::ReverseByteOrder(nFileCode);

	fwrite(&nFileCode, sizeof(nFileCode), 1, pFile);

	__int8 nUnusedBytes[conUnusedBytes] = {0}; // 20 bytes unused.

	fwrite(&nFileCode, conUnusedBytes, 1, pFile);

	// File length is written later but put a value in anyway
	int nLength = 0;

	Utilities::ReverseByteOrder(nLength);

	fwrite(&nLength, sizeof(nLength), 1, pFile);


	// ***********  VERSION CODE  **********
	fwrite(&conShapeFileVersion, sizeof(int), 1, pFile);

	// ***********       TYPE    **********
	fwrite(&conSFPolygon, sizeof(int), 1, pFile);

	return true;
}


/**--------------------------------------------------------------------------<BR>
CShapeFile::
\brief 
<P>---------------------------------------------------------------------------*/
bool CShapeFile::WriteBoundingBox(FILE* pFile, const C2DHoledPolygonSet& aPolygons)
{
	C2DRect rect;
	aPolygons.GetBoundingRect(rect);

	sSFBoundingBox bb = {0};
	bb.Xmin = rect.GetLeft();
	bb.XMax = rect.GetRight();
	bb.Ymin = rect.GetBottom();
	bb.YMax = rect.GetTop();
	fwrite(&bb, sizeof(sSFBoundingBox), 1, pFile);

	return true;
}


/**--------------------------------------------------------------------------<BR>
CShapeFile::
\brief 
<P>---------------------------------------------------------------------------*/
bool CShapeFile::WriteBoundingBox(FILE* pFile, const C2DHoledPolyBaseSet& aPolygons)
{
	C2DRect rect;
	aPolygons.GetBoundingRect(rect);

	sSFBoundingBox bb = {0};
	bb.Xmin = rect.GetLeft();
	bb.XMax = rect.GetRight();
	bb.Ymin = rect.GetBottom();
	bb.YMax = rect.GetTop();
	fwrite(&bb, sizeof(sSFBoundingBox), 1, pFile);

	return true;
}

/**--------------------------------------------------------------------------<BR>
CShapeFile::
\brief 
<P>---------------------------------------------------------------------------*/
bool CShapeFile::WriteBoundingBox(FILE* pFile, const vector<C2DHoledPolygonSet*>& aPolygonsSets)
{
	C2DRect rect;
	if (aPolygonsSets.size() != 0)
	{
		aPolygonsSets[0]->GetBoundingRect(rect);

		C2DRect recttemp;
		for (unsigned int i = 0; i < aPolygonsSets.size(); i++)
		{
			aPolygonsSets[i]->GetBoundingRect(recttemp);
			rect.ExpandToInclude(recttemp);
		}
	}

	sSFBoundingBox bb = {0};
	bb.Xmin = rect.GetLeft();
	bb.XMax = rect.GetRight();
	bb.Ymin = rect.GetBottom();
	bb.YMax = rect.GetTop();
	fwrite(&bb, sizeof(sSFBoundingBox), 1, pFile);

	return true;
}


/**--------------------------------------------------------------------------<BR>
CShapeFile::
\brief 
<P>---------------------------------------------------------------------------*/
void CShapeFile::WriteFileLength(FILE* pFile)
{
	fseek(pFile, 0, SEEK_END);
	int nLength = (int)ftell(pFile) / 2;

	Utilities::ReverseByteOrder(nLength);	

	fseek(pFile, 24, SEEK_SET); // Length is byte 24-28.

	fwrite(&nLength, sizeof(int), 1, pFile);
}


/**--------------------------------------------------------------------------<BR>
CShapeFile::
\brief 
<P>---------------------------------------------------------------------------*/
bool CShapeFile::Write(const CShapeFilePolygon* pPoly, unsigned int nIndex, FILE* pFile)
{
	Utilities::ReverseByteOrder(nIndex);

	fwrite(&nIndex, sizeof(nIndex), 1, pFile);

	unsigned int nRecordLength = pPoly->GetRecordLength();
	
	Utilities::ReverseByteOrder(nRecordLength);

	fwrite(&nRecordLength, sizeof(nRecordLength), 1, pFile);

	unsigned int nShapeType = conSFPolygon;

	//Utilities::ReverseByteOrder(nShapeType);

	// Shape type = 5
	fwrite(&conSFPolygon, sizeof(conSFPolygon), 1, pFile);

	// Bounding box
	fwrite(&pPoly->m_dBox[0], sizeof(double), 1, pFile);
	fwrite(&pPoly->m_dBox[1], sizeof(double), 1, pFile);
	fwrite(&pPoly->m_dBox[2], sizeof(double), 1, pFile);
	fwrite(&pPoly->m_dBox[3], sizeof(double), 1, pFile);

	fwrite(&pPoly->m_nNumParts, INTEGER_BYTES, 1, pFile);
	fwrite(&pPoly->m_nNumPoints, INTEGER_BYTES, 1, pFile);

	fwrite(pPoly->m_pParts, INTEGER_BYTES * pPoly->m_nNumParts, 1, pFile);

	fwrite(pPoly->m_pPoints, POINT_BYTES * pPoly->m_nNumPoints, 1, pFile);

	return true;
}


/**--------------------------------------------------------------------------<BR>
CShapeFile::
\brief 
<P>---------------------------------------------------------------------------*/
void CShapeFile::CreatePolygon(const C2DHoledPolyBase& rPoly, CShapeFilePolygon& rSFPoly)
{
	rSFPoly.m_nNumPoints = rPoly.GetLineCount();

	rSFPoly.m_nNumParts = rPoly.GetHoleCount() + 1; // holes + rim

	rSFPoly.Create();

	rSFPoly.m_pParts[0] = 0;

	sSFPoint* sfPoints = rSFPoly.m_pPoints;

	unsigned int nCurrentPoint = 0;

	WritePoints(sfPoints, nCurrentPoint, rPoly.GetRim(), true);

	for (unsigned int i = 0; i < rPoly.GetHoleCount(); i++)
	{
		rSFPoly.m_pParts[i + 1] = nCurrentPoint;
		WritePoints(sfPoints, nCurrentPoint, rPoly.GetHole(i), false);
	}

	C2DRect rect;
	rPoly.GetBoundingRect(rect);

	// XMin, YMin, XMax, YMax;
	rSFPoly.m_dBox[0] = rect.GetLeft();
	rSFPoly.m_dBox[1] = rect.GetBottom();
	rSFPoly.m_dBox[2] = rect.GetRight();
	rSFPoly.m_dBox[3] = rect.GetTop();
}



/**--------------------------------------------------------------------------<BR>
CShapeFile::
\brief 
<P>---------------------------------------------------------------------------*/
void CShapeFile::CreatePolygon(const C2DHoledPolygonSet& rPolys, CShapeFilePolygon& rSFPoly)
{
	rSFPoly.m_nNumPoints = 0;

	rSFPoly.m_nNumParts = 0;

	for (unsigned int i = 0; i < rPolys.size(); i++)
	{
		rSFPoly.m_nNumPoints += rPolys[i].GetLineCount();

		rSFPoly.m_nNumParts += rPolys[i].GetHoleCount() + 1; // holes + rim
	}

	rSFPoly.Create();

	rSFPoly.m_pParts[0] = 0;

	sSFPoint* sfPoints = rSFPoly.m_pPoints;

	unsigned int nCurrentPoint = 0;
	unsigned int nCurrentPart = 0;

	for (unsigned int p = 0; p < rPolys.size(); p++)
	{
		rSFPoly.m_pParts[nCurrentPart] = nCurrentPoint;
		WritePoints(sfPoints, nCurrentPoint, rPolys[p].GetRim(), true);
		nCurrentPart++;

		for (unsigned int i = 0; i < rPolys[p].GetHoleCount(); i++)
		{
			rSFPoly.m_pParts[nCurrentPart] = nCurrentPoint;
			WritePoints(sfPoints, nCurrentPoint, rPolys[p].GetHole(i), false);
			nCurrentPart++;
		}
	}

	C2DRect rect;
	rPolys.GetBoundingRect(rect);

	// XMin, YMin, XMax, YMax;
	rSFPoly.m_dBox[0] = rect.GetLeft();
	rSFPoly.m_dBox[1] = rect.GetBottom();
	rSFPoly.m_dBox[2] = rect.GetRight();
	rSFPoly.m_dBox[3] = rect.GetTop();
}



/**--------------------------------------------------------------------------<BR>
CShapeFile::
\brief 
<P>---------------------------------------------------------------------------*/
void CShapeFile::WritePoints(sSFPoint* pPoints, unsigned int& nCurrentPoint, 
							 const C2DPolyBase* pPolygon, bool bClockwise)
{
	unsigned int nCount = pPolygon->GetLineCount();

	unsigned int nNumberWritten = 0;
	unsigned int nIterator = 0;

	if (!bClockwise)
		nIterator = nCount - 1;

	while (nNumberWritten < nCount)
	{
		C2DPoint pt = pPolygon->GetLine(nIterator)->GetPointFrom();

		pPoints[nCurrentPoint].dX = pt.x;
		pPoints[nCurrentPoint].dY = pt.y;

		nNumberWritten++;
		nCurrentPoint++;

		bClockwise? nIterator++ : nIterator--;
	}
}